/* -*- java -*-
   $Id: SatoriPosix.java,v 1.1.1.1 2004/02/11 08:01:36 admin Exp $
*/
package satori;
import java.io.*;
import java.util.*;

public class SatoriPosix {
    private int context_id = -1;
    // DO NOT CHANGE THIS AFTER INITIALIZATION

    public static final int VERSION = 124001; // ˺ʤ

    private static String shioribox_path;
    private static String libsatori_path;
    private static String saori_path;

    static {
	System.loadLibrary("satoribridge");
    }

    public SatoriPosix(File ghostdir) {
	if (shioribox_path == null || libsatori_path == null) {
	    throw new RuntimeException(
		"notifyRootDir() hasn't called before the constructor being called. "+
		"It seems this NiseRingo is too old, please update if so.");
	}
	if (saori_path == null) {
	    throw new RuntimeException(
		"notifySaoriPath() hasn't called before the constructor being called. "+
		"It seems this NiseRingo is too old, please update if so.");
	}

	Hashtable additional_env = new Hashtable();
	additional_env.put("SAORI_FALLBACK_PATH", saori_path);
	additional_env.put("SAORI_FALLBACK_ALWAYS", "1");
	
	context_id = construct(
	    shioribox_path,
	    "satori",
	    libsatori_path,
	    additional_env,
	    ghostdir.getAbsolutePath());
    }

    public void terminate() {
	destruct(context_id);
	context_id = -1;
    }

    public String request(String request) {
	// Charsetإåդƥꥯȯԡäȴ
	String temp_str = request.substring(0,request.length()-2) + "Charset: Shift_JIS\r\n\r\n";
	byte[] result;
	try {
	    result = request(context_id, temp_str.getBytes("SJIS"));
	} catch (Exception e) {
	    throw new RuntimeException("Failed to request.", e);
	}

	// 椫Charsetդ롣
	temp_str = new String(result);
	int pos_charset = temp_str.indexOf("Charset:");
	if (pos_charset == -1) {
	    return temp_str;
	} else {
	    int pos_crlf = temp_str.indexOf("\r\n", pos_charset);
	    String charset = temp_str.substring(pos_charset+8, pos_crlf).trim();
	    try {
		return new String(result, charset);
	    } catch (Exception e) {
		throw new RuntimeException(
		    "Failed to parse the response.", e);
	    }
	}
    }

    public static String getModuleName() {
	return "\u91cc\u3005POSIX"; // ΤPOSIX
    }

    public static String getModuleInfo() {
	return "\u91cc\u3005POSIX "+VERSION+" powered by \u91cc\u3005"+
	    "-- http://www.geocities.co.jp/SiliconValley-Cupertino/8536/";
    }

    public static boolean checkIfUsable(File ghost_dir) {
	// satori.dlltrue
	return (new File(ghost_dir,"satori.dll")).exists();
    }
    public static boolean checkIfUsable(File ghost_dir, File shiori_dll) {
	// shiori_dll̾satori.dllʤtrue
	// Ǥʤshiori_dllsatori.dllʸ󤬴ޤޤƤtrue
	if (shiori_dll.getName().equals("satori.dll")) {
	    return true;
	}
	else if (shiori_dll.exists()) {
	    return (findData(shiori_dll,"satori.dll".getBytes()) != -1);
	}
	else {
	    return false;
	}
    }
    // ʲSCMiscUtils饳ԥڡ
    public static int findData(File f,byte[] data) {
	// ե뤫Х򸡺ĤФ줬Ϥޤ륪եåȤ򡢸Ĥʤ-1֤ޤ
	// ߤμǤϥեݤȥɤ߹ǤޤΤǡޤ˵ʥեФƤԲǽǤ
	try {
	    FileInputStream fis = new FileInputStream(f);
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    byte[] buf = new byte[512];
	    while (true) {
		int count = fis.read(buf);
		if (count == -1) break; // eof
		
		baos.write(buf,0,count);
	    }
	    byte[] world = baos.toByteArray();
	    
	    return findData(world,data);
	}
	catch (Exception e) {
	    e.printStackTrace();
	    return -1;
	}
    }
    public static int findData(byte[] world,byte[] data) {
	// world⤫data򸡺ĤФΥեåȤ򡢸Ĥʤ-1֤
	// Boyer MooreˡѤ롣
	int data_len = data.length;
	int[] skip = new int[256];
	for (int i = 0; i < 256; i++) {
	    skip[i] = data_len;
	}
	for (int i = 0; i < data_len-1; i++) {
	    // byte -> -128  +127 ޤǡ
	    // -1255ˡ-2254ˡġ-128128б롣
	    // ĤޤꡢϰϤǤϤ줾(256 + n)б롣
	    skip[(data[i] < 0 ? 256 + data[i] : data[i])] = data_len - i - 1;
	}
	int limit = world.length - data.length;
	for (int i = 0;
	     i <= limit;
	     i += skip[(world[i+data_len-1] < 0 ?
			256 + world[i+data_len-1] : world[i+data_len-1])]) {
	    if (world[i+data_len-1] != data[data_len-1]) {
		continue;
	    }
	    
	    boolean matched = true;
	    for (int j = 0; j < data_len; j++) {
		if (world[i+j] != data[j]) {
		    matched = false;
		    break;
		}
	    }
	    if (matched) {
		return i;
	    }
	}
	return -1;
    }

    public static String getPluginType() {
	return "shiori";
    }

    public static String getPluginName() {
	return "\u91cc\u3005POSIX"; // ΤPOSIX
    }

    public static void notifyRootDir(File root_dir) {
	File shioribox = new File(root_dir, "shioribox");
	shioribox_path = shioribox.getAbsolutePath();

	File libsatori = new File(root_dir, "libsatori.bundle");
	libsatori_path = libsatori.getAbsolutePath();
    }

    public static void notifySaoriPath(String path) {
	saori_path = path; // ڤХѥ
    }

    protected native synchronized int construct(
	String shioribox_path,
	String shiori_name,
	String shiori_module,
	Hashtable additional_env,
	String ghost_path);
    protected native synchronized void destruct(int context_id);
    protected native synchronized byte[] request(int context_id, byte[] req);
}
